@@ -1,3 +1,5 @@
+"""This is the root package of the torchtree framework."""
+from ._version import __version__
 from .core.parameter import CatParameter, Parameter, TransformedParameter, ViewParameter
 
 __all__ = [
