@@ -5,11 +5,13 @@ import logging
 import sys
 from typing import Union
 
-import numpy as np
 import torch
 
 from torchtree import Parameter
+from torchtree.cli import PLUGIN_MANAGER
+from torchtree.cli.argparse_utils import list_or_int
 from torchtree.cli.evolution import (
+    COALESCENT_PIECEWISE,
     create_alignment,
     create_evolution_joint,
     create_evolution_parser,
@@ -18,6 +20,7 @@ from torchtree.cli.evolution import (
     create_taxa,
 )
 from torchtree.cli.jacobians import create_jacobians
+from torchtree.cli.utils import CONSTRAINT
 from torchtree.distributions import Distribution
 
 logger = logging.getLogger(__name__)
@@ -51,13 +54,13 @@ def create_variational_parser(subprasers):
     )
     parser.add_argument(
         '--elbo_samples',
-        type=int,
+        type=list_or_int,
         default=100,
         help="""number of samples for Monte Carlo estimate of ELBO""",
     )
     parser.add_argument(
         '--grad_samples',
-        type=int,
+        type=list_or_int,
         default=1,
         help="""number of samples for Monte Carlo estimate of gradients""",
     )
@@ -68,6 +71,13 @@ def create_variational_parser(subprasers):
         help="number of samples for Monte Carlo estimate of gradients"
         " using multisample objective",
     )
+    parser.add_argument(
+        '--K_elbo_samples',
+        type=int,
+        default=1,
+        help="number of samples for Monte Carlo estimate of ELBO"
+        " using multisample objective",
+    )
     parser.add_argument(
         '--samples',
         type=int,
@@ -110,6 +120,17 @@ def create_variational_parser(subprasers):
         default=100,
         help="""convergence every N iterations""",
     )
+    parser.add_argument(
+        '--divergence',
+        choices=['ELBO', 'KLpq'],
+        default='ELBO',
+        help="""divergence to optimize""",
+    )
+    parser.add_argument(
+        "--checkpoint_all",
+        action='store_true',
+        help="""log a new checkpoint file periodically""",
+    )
     parser.set_defaults(func=build_advi)
     return parser
 
@@ -126,9 +147,10 @@ def _unique_id(id_, dic):
 def create_tril(scales: torch.Tensor) -> torch.Tensor:
     """Create a 1 dimentional tensor containing a flatten tridiagonal matrix.
 
-    A covariance matrix is created using scales**2 for variances and the covariances
-    are set to zero. A tridiagonal is created using the cholesky decomposition and the
-    diagonal elements are replaced with their log.
+    A covariance matrix is created using scales**2 for variances and the
+    covariances are set to zero. A tridiagonal is created using the
+    cholesky decomposition and the diagonal elements are replaced with
+    their log.
 
     :param scales: standard deviations
     :return:
@@ -252,27 +274,32 @@ def create_flexible_variational(arg, json_object):
                 )
                 joint_var.extend(distr)
                 var_parameters.extend(var_params)
+        elif id_.lower().startswith('realnvp'):
+            var_id = f"var.{id_}"
+            res = apply_transforms_for_fullrank(id_, params)
+            x = list(map(lambda x: x[1], res))
+            params = torch.cat(list(map(lambda x: torch.tensor(x[2]), res)))
+
+            distr = create_realnp_distribution(var_id, x, params)
+            var_parameters.append(var_id)
+            joint_var.append(distr)
     return joint_var, var_parameters
 
 
-def create_realnvp(var_id, json_object, arg):
-    group_map = {}
-    parameters = {'UNSPECIFIED': []}
-    gather_parameters(json_object, group_map, parameters)
-    res = apply_transforms_for_fullrank(var_id, parameters['UNSPECIFIED'])
-    x = list(map(lambda x: x[1], res))
-
-    params = torch.cat(list(map(lambda x: torch.tensor(x[2]), res)))
-
-    distr = {
+def create_realnp_distribution(var_id: str, x, params: torch.Tensor):
+    return {
         "id": var_id,
         "type": "RealNVP",
         "x": x,
         "base": {
-            "id": "base",
+            "id": f"{var_id}.base",
             "type": "Distribution",
             "distribution": "torchtree.distributions.Normal",
-            "x": {"id": "dummy", "type": "Parameter", "zeros": params.shape[-1]},
+            "x": {
+                "id": f"{var_id}.dummy",
+                "type": "Parameter",
+                "zeros": params.shape[-1],
+            },
             "parameters": {
                 "loc": {
                     "id": f"{var_id}.base.loc",
@@ -290,6 +317,18 @@ def create_realnvp(var_id, json_object, arg):
         "hidden_size": 2,
         "n_hidden": 1,
     }
+
+
+def create_realnvp(var_id, json_object, arg):
+    group_map = {}
+    parameters = {'UNSPECIFIED': []}
+    gather_parameters(json_object, group_map, parameters)
+    res = apply_transforms_for_fullrank(var_id, parameters['UNSPECIFIED'])
+    x = list(map(lambda x: x[1], res))
+
+    params = torch.cat(list(map(lambda x: torch.tensor(x[2]), res)))
+
+    distr = create_realnp_distribution(var_id, x, params)
     var_parameters = (var_id,)
     return distr, var_parameters
 
@@ -301,9 +340,10 @@ def gather_parameters(json_object: dict, group_map: dict, parameters: dict):
     elif isinstance(json_object, dict):
         if 'type' in json_object and json_object['type'] == 'Parameter':
             if (
-                'lower' not in json_object
-                or 'upper' not in json_object
-                or json_object['lower'] != json_object['upper']
+                CONSTRAINT.LOWER.value not in json_object
+                or CONSTRAINT.UPPER.value not in json_object
+                or json_object[CONSTRAINT.LOWER.value]
+                != json_object[CONSTRAINT.UPPER.value]
             ):
                 if json_object['id'] in group_map:
                     parameters[group_map[json_object['id']]].append(json_object)
@@ -367,9 +407,9 @@ def apply_affine_transform(json_object, loc, scale):
     json_object['x'] = {
         'id': unshifted_id,
         'type': 'Parameter',
-        'tensor': (np.array(json_object['tensor']) - loc).tolist(),
+        'tensor': (torch.tensor(json_object['tensor']) - loc).tolist(),
     }
-    json_object['x']['lower'] = 0
+    json_object['x'][CONSTRAINT.LOWER.value] = 0
     del json_object['tensor']
     return unshifted_id
 
@@ -381,7 +421,7 @@ def apply_exp_transform(json_object):
     json_object['x'] = {
         'id': unres_id,
         'type': 'Parameter',
-        'tensor': np.log(json_object['tensor']).tolist(),
+        'tensor': torch.tensor(json_object['tensor']).log().tolist(),
     }
     if 'full' in json_object:
         json_object['x']['full'] = json_object['full']
@@ -466,6 +506,12 @@ def create_gamma_distribution(var_id, x_unres, json_object, concentration, rate)
             **{'full_like': json_object['id'], 'tensor': rate},
         ),
     }
+
+    if isinstance(concentration, list):
+        del concentration_param["x"]["full_like"]
+    if isinstance(rate, list):
+        del rate_param["x"]["full_like"]
+
     distr = Distribution.json_factory(
         var_id + '.' + json_object['id'],
         'torch.distributions.Gamma',
@@ -516,8 +562,14 @@ def create_meanfield(
             var_parameters.extend(params)
     elif isinstance(json_object, dict):
         if 'type' in json_object and json_object['type'] == 'Parameter':
-            if 'lower' in json_object and 'upper' in json_object:
-                if json_object['lower'] != json_object['upper']:
+            if (
+                CONSTRAINT.LOWER.value in json_object
+                and CONSTRAINT.UPPER.value in json_object
+            ):
+                if (
+                    json_object[CONSTRAINT.LOWER.value]
+                    != json_object[CONSTRAINT.UPPER.value]
+                ):
                     apply_sigmoid_transformed(json_object)
                     distrs, params = create_meanfield(
                         var_id, json_object['x'], distribution
@@ -525,9 +577,11 @@ def create_meanfield(
                     distributions.extend(distrs)
                     var_parameters.extend(params)
                     return distributions, var_parameters
-            elif 'lower' in json_object:
-                if json_object['lower'] > 0:
-                    apply_affine_transform(json_object, json_object['lower'], 1.0)
+            elif CONSTRAINT.LOWER.value in json_object:
+                if json_object[CONSTRAINT.LOWER.value] > 0:
+                    apply_affine_transform(
+                        json_object, json_object[CONSTRAINT.LOWER.value], 1.0
+                    )
 
                     # now id becomes id.unshifted with a lower bound of 0 so another
                     # round of create_meanfield to create a id.unshifted.unres
@@ -540,10 +594,11 @@ def create_meanfield(
                     var_parameters.extend(params)
                     return distributions, var_parameters
                 elif distribution == 'Normal':
-                    tensor = np.array(json_object['tensor'])
-                    loc = np.log(tensor / np.sqrt(1 + 0.001 / tensor**2)).tolist()
-                    scale_log = np.log(
-                        np.sqrt(np.log(1 + 0.001 / tensor**2))
+                    tensor = torch.tensor(json_object['tensor'])
+                    var = tensor * 0.01
+                    loc = torch.log(tensor / torch.sqrt(1 + var / tensor**2)).tolist()
+                    scale_log = torch.log(
+                        torch.sqrt(torch.log(1 + var / tensor**2))
                     ).tolist()
                     unres_id = apply_exp_transform(json_object)
                     distr, loc, scale = create_normal_distribution(
@@ -552,8 +607,17 @@ def create_meanfield(
                     var_parameters.extend((loc['id'], scale['x']['id']))
                 elif distribution in ('Gamma', 'Weibull'):
                     if distribution == 'Gamma':
+                        # solve mean \alpha/\beta=tensor and
+                        # variance \alpha/\beta^2=0.01*tensor
+                        tensor = torch.tensor(json_object["tensor"])
+                        log_concentration = (tensor / 0.01).log().tolist()
+                        log_rate = torch.log(1.0 / 0.01).tolist()
                         distr, concentration, rate = create_gamma_distribution(
-                            var_id, json_object['id'], json_object, 0, 2.3
+                            var_id,
+                            json_object['id'],
+                            json_object,
+                            log_concentration,
+                            log_rate,
                         )
                     elif distribution == 'Weibull':
                         distr, scale, concentration = create_weibull_distribution(
@@ -564,7 +628,7 @@ def create_meanfield(
                     )
                 distributions.append(distr)
                 parameters.append(json_object['id'])
-            elif 'simplex' in json_object and json_object['simplex']:
+            elif json_object.get(CONSTRAINT.SIMPLEX.value, False):
                 unres_id = apply_simplex_transform(json_object)
                 distr, loc, scale = create_normal_distribution(
                     var_id, unres_id, json_object, 0.5, -1.89712
@@ -573,7 +637,7 @@ def create_meanfield(
                 var_parameters.extend((loc['id'], scale['x']['id']))
                 parameters.append(json_object['id'])
             else:
-                tensor = np.array(json_object['tensor'])
+                tensor = torch.tensor(json_object['tensor'])
                 distr, loc, scale = create_normal_distribution(
                     var_id,
                     json_object['id'],
@@ -603,8 +667,14 @@ def apply_transforms_for_fullrank(
             var_parameters.extend(params)
     elif isinstance(json_object, dict):
         if 'type' in json_object and json_object['type'] == 'Parameter':
-            if 'lower' in json_object and 'upper' in json_object:
-                if json_object['lower'] != json_object['upper']:
+            if (
+                CONSTRAINT.LOWER.value in json_object
+                and CONSTRAINT.UPPER.value in json_object
+            ):
+                if (
+                    json_object[CONSTRAINT.LOWER.value]
+                    != json_object[CONSTRAINT.UPPER.value]
+                ):
                     unres_id = apply_sigmoid_transformed(json_object)
                     tensor_list = json_object['x']['tensor']
                     # full is list representing the shape/length of the tensor
@@ -614,13 +684,15 @@ def apply_transforms_for_fullrank(
                             json_object['x']['full'] * json_object['x']['tensor']
                         )
                     var_parameters.append((json_object['id'], unres_id, tensor_list))
-            elif 'lower' in json_object:
-                if json_object['lower'] > 0:
-                    apply_affine_transform(json_object, json_object['lower'], 1.0)
+            elif CONSTRAINT.LOWER.value in json_object:
+                if json_object[CONSTRAINT.LOWER.value] > 0:
+                    apply_affine_transform(
+                        json_object, json_object[CONSTRAINT.LOWER.value], 1.0
+                    )
                     unres_id = apply_exp_transform(json_object['x'])
                     tensor_list = json_object['x']['x']['tensor']
                     var_parameters.append((json_object['id'], unres_id, tensor_list))
-                elif json_object['lower'] == 0:
+                elif json_object[CONSTRAINT.LOWER.value] == 0:
                     unres_id = apply_exp_transform(json_object)
                     tensor_list = json_object['x']['tensor']
                     if 'full' in json_object['x']:
@@ -630,7 +702,7 @@ def apply_transforms_for_fullrank(
                     var_parameters.append((json_object['id'], unres_id, tensor_list))
                 else:
                     raise NotImplementedError
-            elif 'simplex' in json_object and json_object['simplex']:
+            elif json_object.get(CONSTRAINT.SIMPLEX.value, False):
                 unres_id = apply_simplex_transform(json_object)
                 tensor_list = json_object['x']['tensor']
                 if 'full' in json_object['x']:
@@ -657,7 +729,9 @@ def apply_transforms_for_fullrank(
 
 def create_variational_model(id_, joint, arg) -> tuple[dict, list[str]]:
     variational = {'id': id_, 'type': 'JointDistributionModel'}
-    if len(arg.variational) == 1 and arg.variational[0] == 'meanfield':
+    if isinstance(arg.variational, str) or (
+        len(arg.variational) == 1 and arg.variational[0] == 'meanfield'
+    ):
         distributions, parameters = create_meanfield(id_, joint, arg.distribution)
     elif (
         arg.variational == 'fullrank'
@@ -683,40 +757,102 @@ def create_advi(joint, variational, parameters, arg):
         checkpoint = arg.stem + '-checkpoint.json'
     else:
         checkpoint = 'checkpoint.json'
-    advi_dic = {
-        'id': 'advi',
-        'type': 'Optimizer',
-        'algorithm': 'torch.optim.Adam',
-        'options': {'lr': arg.lr},
-        'maximize': True,
-        'checkpoint': checkpoint,
-        'iterations': arg.iter,
-        'loss': {
+
+    if arg.divergence == 'ELBO':
+        loss = {
             'id': 'elbo',
             'type': 'ELBO',
             'samples': grad_samples,
             'joint': joint,
             'variational': variational,
-        },
+        }
+        maximize = True
+    elif arg.divergence == 'KLpq':
+        loss = {
+            'id': 'elbo',
+            'type': 'KLpqImportance',
+            'samples': grad_samples,
+            'joint': joint,
+            'variational': variational,
+        }
+        maximize = False
+    advi_dic = {
+        'id': 'advi',
+        'type': 'Optimizer',
+        'algorithm': 'torch.optim.Adam',
+        'options': {'lr': arg.lr},
+        'maximize': maximize,
+        'checkpoint': checkpoint,
+        'iterations': arg.iter,
+        'loss': loss,
         'parameters': parameters,
     }
 
-    advi_dic['convergence'] = {
-        'type': 'StanVariationalConvergence',
-        'max_iterations': arg.iter,
-        'loss': 'elbo',
-        'every': arg.convergence_every,
-        'samples': arg.elbo_samples,
-        'tol_rel_obj': arg.tol_rel_obj,
-    }
-    advi_dic['scheduler'] = {
-        'type': 'torchtree.optim.Scheduler',
-        'scheduler': 'torch.optim.lr_scheduler.LambdaLR',
-        'lr_lambda': 'lambda epoch: 1.0 / (epoch + 1)**0.5',
-    }
+    if arg.checkpoint_all:
+        advi_dic["checkpoint_all"] = True
+
+    if arg.K_elbo_samples > 1:
+        elbo_samples = [arg.elbo_samples, arg.K_elbo_samples]
+    else:
+        elbo_samples = arg.elbo_samples
+
+    if isinstance(elbo_samples, list) or elbo_samples > 0:
+        if arg.divergence == 'ELBO':
+            loss2 = 'elbo'
+        elif arg.divergence == 'KLpq':
+            loss2 = {
+                'id': 'loss2',
+                'type': 'KLpq',
+                'samples': grad_samples,
+                'joint': joint,
+                'variational': variational,
+            }
+
+        advi_dic['convergence'] = {
+            'type': 'StanVariationalConvergence',
+            'max_iterations': arg.iter,
+            'loss': loss2,
+            'every': arg.convergence_every,
+            'samples': elbo_samples,
+            'tol_rel_obj': arg.tol_rel_obj,
+        }
+        advi_dic['scheduler'] = {
+            'type': 'torchtree.optim.Scheduler',
+            'scheduler': 'torch.optim.lr_scheduler.LambdaLR',
+            'lr_lambda': 'lambda epoch: 1.0 / (epoch + 1)**0.5',
+        }
     return advi_dic
 
 
+def create_logger(id_, parameters, arg):
+    if arg.stem:
+        file_name = arg.stem + '-samples.csv'
+    else:
+        file_name = 'samples.csv'
+
+    parameters2 = list(filter(lambda x: 'tree.ratios' != x, parameters))
+    models = ['joint', 'like', 'prior']
+    if arg.coalescent:
+        models.append('coalescent')
+        if arg.coalescent in COALESCENT_PIECEWISE:
+            models.append('gmrf')
+            models.append(
+                {
+                    'id': arg.coalescent,
+                    'type': 'JointDistributionModel',
+                    'distributions': ['coalescent', 'gmrf'],
+                }
+            )
+
+    return {
+        "id": id_,
+        "type": "Logger",
+        "file_name": file_name,
+        "parameters": models + parameters2 + ['coalescent.theta.log'],
+        "delimiter": "\t",
+    }
+
+
 def create_sampler(id_, var_id, parameters, arg):
     if arg.stem:
         file_name = arg.stem + '-samples.csv'
@@ -726,6 +862,24 @@ def create_sampler(id_, var_id, parameters, arg):
         tree_file_name = 'samples.trees'
 
     parameters2 = list(filter(lambda x: 'tree.ratios' != x, parameters))
+    models = ['joint.jacobian', 'joint', 'like', 'prior', var_id]
+
+    if arg.location_regex:
+        models.append('like.location')
+    elif arg.metadata and arg.trait:
+        models.extend([f'like.{trait}' for trait in arg.trait])
+
+    if arg.coalescent:
+        models.append('coalescent')
+        if arg.coalescent in COALESCENT_PIECEWISE:
+            models.append('gmrf')
+            models.append(
+                {
+                    'id': arg.coalescent,
+                    'type': 'JointDistributionModel',
+                    'distributions': ['coalescent', 'gmrf'],
+                }
+            )
 
     return {
         "id": id_,
@@ -737,7 +891,7 @@ def create_sampler(id_, var_id, parameters, arg):
                 "id": "logger",
                 "type": "Logger",
                 "file_name": file_name,
-                "parameters": ['joint', 'like', var_id] + parameters2,
+                "parameters": models + parameters2,
                 "delimiter": "\t",
             },
             {
@@ -777,14 +931,24 @@ def build_advi(arg):
     jacobians_list = create_jacobians(json_list)
     if arg.clock is not None and arg.heights == 'ratio':
         jacobians_list.append('tree')
-    if arg.coalescent in ('skygrid', 'skyride'):
+
+    if arg.coalescent in COALESCENT_PIECEWISE:
         jacobians_list.remove("coalescent.theta")
-    joint_dic['distributions'].extend(jacobians_list)
+
+    joint_jacobian = {
+        'id': 'joint.jacobian',
+        'type': 'JointDistributionModel',
+        'distributions': ['joint'] + jacobians_list,
+    }
+    json_list.append(joint_jacobian)
 
     json_list.append(var_dic)
 
-    advi_dic = create_advi('joint', 'variational', var_parameters, arg)
-    json_list.append(advi_dic)
+    # only create a sampler
+    # useful when we have a checkpoint.json file and we only want to sample
+    if arg.iter > 0:
+        advi_dic = create_advi('joint.jacobian', 'variational', var_parameters, arg)
+        json_list.append(advi_dic)
 
     parameters = []
     if arg.clock is not None:
@@ -792,7 +956,24 @@ def build_advi(arg):
         if arg.heights == 'ratio':
             parameters.extend(["tree.ratios", "tree.root_height"])
         elif arg.heights == 'shift':
-            parameters.extend(["tree.heights"])
+            parameters.extend(
+                [
+                    'tree.shifts',
+                    {
+                        'id': "tree.root_height",
+                        'type': 'ViewParameter',
+                        'indices': '-1:',
+                        'parameter': {
+                            'id': 'tree.heights',
+                            'type': 'TransformedParameter',
+                            'transform': 'torchtree.evolution.tree_height_transform'
+                            '.DifferenceNodeHeightTransform',
+                            'x': 'tree.shifts',
+                            'parameters': {'tree_model': 'tree'},
+                        },
+                    },
+                ]
+            )
 
         if arg.clock == 'ucln':
             parameters.extend(
@@ -810,12 +991,14 @@ def build_advi(arg):
         parameters = ['tree.blens']
 
     if arg.coalescent is not None:
-        parameters.append("coalescent.theta")
-        if arg.coalescent in ('skygrid', 'skyride'):
+        if arg.coalescent_integrated is None:
+            parameters.append("coalescent.theta")
+
+        if arg.coalescent in COALESCENT_PIECEWISE and not arg.gmrf_integrated:
             parameters.append('gmrf.precision')
         elif arg.coalescent == 'exponential':
             parameters.append('coalescent.growth')
-        elif arg.coalescent == 'piecewise':
+        if arg.coalescent == 'piecewise-exponential':
             parameters.append('coalescent.growth')
     elif arg.birth_death is not None:
         parameters.append("bdsk.R")
@@ -828,8 +1011,12 @@ def build_advi(arg):
             parameters.extend(
                 [f"substmodel.{tag}.kappa", f"substmodel.{tag}.frequencies"]
             )
+    elif arg.model == "SYM":
+        parameters.append("substmodel.rates")
     elif arg.model == 'GTR':
         parameters.extend(["substmodel.rates", "substmodel.frequencies"])
+    elif arg.model == 'K80':
+        parameters.append("substmodel.kappa")
     elif arg.model == 'HKY':
         parameters.extend(["substmodel.kappa", "substmodel.frequencies"])
     elif arg.model == 'MG94':
@@ -844,7 +1031,20 @@ def build_advi(arg):
                 parameters.append(f"sitemodel.{tag}.shape")
         else:
             parameters.append("sitemodel.shape")
+    if arg.invariant:
+        if arg.model == 'SRD06':
+            for tag in ('12', '3'):
+                parameters.append(f"sitemodel.{tag}.pinv")
+        else:
+            parameters.append("sitemodel.pinv")
 
     if arg.samples > 0:
         json_list.append(create_sampler('sampler', 'variational', parameters, arg))
+
+    if arg.samples == 0 and arg.iter == 0:
+        json_list.append(create_logger('logger', parameters, arg))
+
+    for plugin in PLUGIN_MANAGER.plugins():
+        plugin.process_all(arg, json_list)
+
     return json_list
