@@ -14,7 +14,7 @@ from .model import CallableModel
 from .parameter_encoder import ParameterEncoder
 from .runnable import Runnable
 from .serializable import JSONSerializable
-from .utils import process_object, process_objects, register_class
+from .utils import JSONParseError, process_object, process_objects, register_class
 
 
 class LoggerInterface(JSONSerializable, Runnable):
@@ -34,14 +34,13 @@ class LoggerInterface(JSONSerializable, Runnable):
 
     def run(self) -> None:
         self.initialize()
-        self.log()
+        self.log(RUN=True)
         self.close()
 
 
 @register_class
 class Logger(LoggerInterface):
-    r"""
-    Class for logging Parameter objects to a file.
+    r"""Class for logging Parameter objects to a file.
 
     :param objs: list of Parameter or CallableModel objects
     :type objs: list[Parameter or CallableModel]
@@ -80,6 +79,20 @@ class Logger(LoggerInterface):
         self.writer.writerow(header)
 
     def log(self, *args, **kwargs) -> None:
+        if kwargs.get('RUN', False):
+            data = [torch.zeros([1])]
+            for obj in self.objs:
+                if isinstance(obj, AbstractParameter):
+                    data.append(obj.tensor.detach().cpu())
+                else:
+                    log_p = obj()
+                    if log_p.dim() == 0:
+                        log_p = log_p.unsqueeze(-1)
+                    data.append(log_p)
+            data = torch.cat(data, -1).tolist()
+            self.writer.writerow(data)
+            return
+
         sample = kwargs.get('sample', self.sample)
         self.sample += 1
         if sample % self.every != 0:
@@ -107,11 +120,11 @@ class Logger(LoggerInterface):
 
         :param data: json representation of Logger object.
         :type data: dict[str,Any]
-        :param dic: dictionary containing additional objects that can be referenced
-         in data.
+        :param dic: dictionary containing additional objects that can be
+                referenced in data.
         :type dic: dict[str,Any]
-
-        :return: a :class:`~torchtree.core.logger.Logger` object.
+        :return: a
+        :class: `~torchtree.core.logger.Logger` object.
         :rtype: Logger
         """
         params = process_objects(data['parameters'], dic)
@@ -134,6 +147,7 @@ class TreeLogger(LoggerInterface):
 
     def __init__(self, tree_model: TreeModel, every: int, **kwargs) -> None:
         self.tree_model = tree_model
+        self.every = every
         self.file_name = kwargs.get('file_name', None)
         self.kwargs = kwargs
         self.sample = 1
@@ -179,16 +193,15 @@ class TreeLogger(LoggerInterface):
 
     @classmethod
     def from_json(cls, data, dic) -> TreeLogger:
-        r"""
-        Create a TreeLogger object.
+        r"""Create a TreeLogger object.
 
         :param data: json representation of TreeLogger object.
         :type data: dict[str,Any]
-        :param dic: dictionary containing additional objects that can be referenced
-         in data.
+        :param dic: dictionary containing additional objects that can be
+                referenced in data.
         :type dic: dict[str,Any]
-
-        :return: a :class:`~torchtree.core.logger.TreeLogger` object.
+        :return: a
+        :class: `~torchtree.core.logger.TreeLogger` object.
         :rtype: TreeLogger
         """
         tree = process_object(data['tree_model'], dic)
@@ -202,8 +215,7 @@ class TreeLogger(LoggerInterface):
 
 @register_class
 class CSV(JSONSerializable, Runnable):
-    r"""
-    Class for writting parameters to a CSV file.
+    r"""Class for writting parameters to a CSV file.
 
     :param objs: list of Parameter objects
     :type objs: list[Parameter]
@@ -229,16 +241,15 @@ class CSV(JSONSerializable, Runnable):
 
     @classmethod
     def from_json(cls, data, dic) -> CSV:
-        r"""
-        Create a CSV object.
+        r"""Create a CSV object.
 
         :param data: json representation of CSV object.
         :type data: dict[str,Any]
-        :param dic: dictionary containing additional objects that can be referenced
-         in data.
+        :param dic: dictionary containing additional objects that can be
+                referenced in data.
         :type dic: dict[str,Any]
-
-        :return: a :class:`~torchtree.core.logger.CSV` object.
+        :return: a
+        :class: `~torchtree.core.logger.CSV` object.
         :rtype: CSV
         """
         params = process_objects(data['parameters'], dic)
@@ -251,8 +262,7 @@ class CSV(JSONSerializable, Runnable):
 
 @register_class
 class Dumper(JSONSerializable, Runnable):
-    r"""
-    Class for saving parameters to a json file.
+    r"""Class for saving parameters to a json file.
 
     :param parameters: list of Parameters.
     :type parameters: list[Parameter]
@@ -268,9 +278,7 @@ class Dumper(JSONSerializable, Runnable):
         self.parameters = parameters
 
     def run(self) -> None:
-        r"""
-        Write the parameters to the file.
-        """
+        r"""Write the parameters to the file."""
         if self.file_name is not None:
             with open(self.file_name, 'w') as fp:
                 json.dump(self.parameters, fp, cls=ParameterEncoder, **self.kwargs)
@@ -279,16 +287,15 @@ class Dumper(JSONSerializable, Runnable):
 
     @classmethod
     def from_json(cls, data, dic) -> Dumper:
-        r"""
-        Create a Dumper object.
+        r"""Create a Dumper object.
 
         :param data: json representation of Dumper object.
         :type data: dict[str,Any]
-        :param dic: dictionary containing additional objects that can be referenced
-         in data.
+        :param dic: dictionary containing additional objects that can be
+                referenced in data.
         :type dic: dict[str,Any]
-
-        :return: a :class:`~torchtree.core.logger.Dumper` object.
+        :return: a
+        :class: `~torchtree.core.logger.Dumper` object.
         :rtype: Dumper
         """
         parameters = process_objects(data['parameters'], dic)
@@ -297,3 +304,76 @@ class Dumper(JSONSerializable, Runnable):
             if key in data:
                 kwargs[key] = data[key]
         return cls(parameters, **kwargs)
+
+
+@register_class
+class ContainerLogger(LoggerInterface):
+    r"""Class for logging Parameter and CallableModel values to a list.
+
+    :param inputs: list of Parameter or CallableModel objects
+    :type inputs: list[Parameter or CallableModel]
+    :param int every: logging frequency
+    """
+
+    def __init__(
+        self,
+        inputs: list[Union[AbstractParameter, CallableModel]],
+        container,
+        every: int,
+    ) -> None:
+        self.container = container
+        self.every = every
+        self.inputs = inputs
+        self.sample = 1
+
+    def initialize(self) -> None:
+        pass
+
+    def log(self, *args, **kwargs) -> None:
+        sample = kwargs.get('sample', self.sample)
+        self.sample += 1
+        if sample % self.every != 0:
+            return
+
+        row = []
+        for obj in self.inputs:
+            if isinstance(obj, AbstractParameter):
+                row.extend(obj.tensor.detach().cpu().tolist())
+            else:
+                log_p = obj()
+                if len(log_p.shape) == 0 or log_p.shape[-1] <= 1:
+                    row.append(log_p.item())
+                else:
+                    row.append(log_p.sum(-1).item())
+        if isinstance(self.container, list):
+            self.container.append(row)
+
+    def close(self) -> None:
+        pass
+
+    @classmethod
+    def from_json(cls, data, dic) -> ContainerLogger:
+        r"""Create a ContainerLogger object.
+
+        :param dict[str, Any] data: dictionary representation of a ContainerLogger
+            object.
+        :param dict[str, Identifiable] dic: dictionary containing torchtree objects
+            keyed by their ID.
+
+        **JSON attributes**:
+
+         Mandatory:
+          - container (list): python list to log values.
+          - inputs (list[AbstractParameter or CallableObject]): list of parameters
+            or models to log.
+
+         Optional:
+          - every (int): logging frequency.
+        """
+        inputs = process_objects(data["inputs"], dic)
+        if isinstance(data["container"], list):
+            container = data["container"]
+        else:
+            raise JSONParseError("The container must be a list")
+        every = data.get("every", 1)
+        return cls(inputs, container, every)
