@@ -1,3 +1,4 @@
+"""Parametric models."""
 import abc
 import collections.abc
 from typing import Optional, Union
@@ -6,13 +7,19 @@ import torch.distributions
 from torch import Tensor
 
 from torchtree.core.abstractparameter import AbstractParameter
-
-from .classproperty_decorator import classproperty
-from .identifiable import Identifiable
-from .parametric import ModelListener, ParameterListener, Parametric
+from torchtree.core.classproperty_decorator import classproperty
+from torchtree.core.identifiable import Identifiable
+from torchtree.core.parametric import ModelListener, ParameterListener, Parametric
 
 
 class Model(Parametric, Identifiable, ModelListener, ParameterListener):
+    """Parametric model.
+
+    A Model can contain parameters and models and can monitor any
+    changes. A Model is the building block of more complex models. This
+    class is abstract.
+    """
+
     _tag = None
 
     def __init__(self, id_: Optional[str]) -> None:
@@ -41,34 +48,50 @@ class Model(Parametric, Identifiable, ModelListener, ParameterListener):
         return cls._tag
 
     def to(self, *args, **kwargs) -> None:
+        """Performs Tensor dtype and/or device conversion using torch.to."""
         self._apply(lambda x: x.to(*args, **kwargs))
 
     def cuda(self, device: Optional[Union[int, torch.device]] = None) -> None:
+        """Move tensors to CUDA using torch.cuda."""
         self._apply(lambda x: x.cuda(device))
 
     def cpu(self) -> None:
+        """Move tensors to CPU memory using ~torch.cpu."""
         self._apply(lambda x: x.cpu())
 
     def _apply(self, fn):
-        for param in self._parameters:
+        for param in self._parameters.values():
             fn(param)
-        for model in self._models:
+        for model in self._models.values():
             fn(model)
 
     def models(self):
+        """Returns sub-models."""
         for model in self._models.values():
             yield model
 
     @property
-    @abc.abstractmethod
     def sample_shape(self) -> torch.Size:
+        """Returns sample shape."""
+        return self._sample_shape()
+
+    @abc.abstractmethod
+    def _sample_shape(self) -> torch.Size:
+        """Implementation of sample_shape.
+
+        :return: sample shape
+        """
         ...
 
 
 class CallableModel(Model, collections.abc.Callable):
-    """Classes extending CallableModel are Callable and the returned value is
-    cached in case we need to use this value multiple times without the need to
-    recompute it."""
+    """Classes inheriting from :class:`Model` and
+    :class:`collections.abc.Callable`.
+
+    CallableModel are Callable and the returned value is cached in case
+    we need to use this value multiple times without the need to
+    recompute it.
+    """
 
     def __init__(self, id_: Optional[str]) -> None:
         Model.__init__(self, id_)
