@@ -1,18 +1,19 @@
+"""Implementation of Parameter classes."""
 from __future__ import annotations
 
 import collections.abc
 import inspect
 import numbers
-from typing import Optional, Union, overload
+from typing import Any, Optional, Union, overload
 
-import numpy as np
 import torch
 from torch import Tensor, nn
 
-from .abstractparameter import AbstractParameter
-from .container import Container
-from .parametric import ParameterListener, Parametric
-from .utils import (
+from torchtree.core.abstractparameter import AbstractParameter
+from torchtree.core.container import Container
+from torchtree.core.identifiable import Identifiable
+from torchtree.core.parametric import ParameterListener, Parametric
+from torchtree.core.utils import (
     JSONParseError,
     get_class,
     process_object,
@@ -24,6 +25,13 @@ from .utils import (
 
 @register_class
 class Parameter(AbstractParameter):
+    """Parameter class.
+
+    :param id_: identifier of Parameter object.
+    :type id_: str or None
+    :param Tensor tensor: Tensor object.
+    """
+
     def __init__(self, id_: Optional[str], tensor: Tensor) -> None:
         super().__init__(id_)
         self._tensor = tensor
@@ -62,16 +70,28 @@ class Parameter(AbstractParameter):
 
     @property
     def grad_fn(self):
+        """The grad_fn property of the tensor.
+
+        :rtype: torch.autograd.graph.node
+        """
         return self._tensor.grad_fn
 
     def copy_(self, tensor):
         self._tensor.copy_(tensor)
 
-    def size(self):
+    def size(self) -> torch.Size:
+        """Returns the size of the tensor.
+
+        :rtype: Size
+        """
         return self._tensor.size()
 
     @property
-    def grad(self) -> bool:
+    def grad(self) -> Tensor:
+        """The grad property of the Tensor.
+
+        :rtype: Tensor
+        """
         return self._tensor.grad
 
     def add_parameter_listener(self, listener) -> None:
@@ -127,7 +147,7 @@ class Parameter(AbstractParameter):
         """
         if len(args) == 0:
             self._tensor = self._tensor.to(
-                device=kwargs['device'], dtype=kwargs['dtype']
+                device=kwargs.get("device", None), dtype=kwargs.get("dtype", None)
             )
         else:
             self._tensor = self._tensor.to(args[0])
@@ -154,6 +174,8 @@ class Parameter(AbstractParameter):
             parameter['ones'] = kwargs['ones']
         elif 'eye' in kwargs:
             parameter['eye'] = kwargs['eye']
+        elif 'eye_like' in kwargs:
+            parameter['eye_like'] = kwargs['eye_like']
         elif 'tensor' in kwargs:
             parameter['tensor'] = kwargs['tensor']
 
@@ -164,7 +186,86 @@ class Parameter(AbstractParameter):
         return parameter
 
     @classmethod
-    def from_json(cls, data, dic):
+    def from_json(cls, data: dict[str, Any], dic: dict[str, Identifiable]) -> Parameter:
+        r"""Creates a Parameter object from a dictionary.
+
+        :param dict[str, Any] data: dictionary representation of a parameter object.
+        :param dict[str, Identifiable] dic: dictionary containing torchtree objects
+            keyed by their ID.
+
+        **JSON attributes**:
+
+         Only one of ``tensor``, ``full_like``, ``full``, ``zeros_like``, ``zeros``,
+         ``ones_like``, ``ones``, ``eye_like``, ``eye``, ``arange`` can be specified.
+
+         - tensor (list): list of scalars.
+         - full_like (str or dict): parameter used to determine the size of
+           the tensor.
+
+           - value (float or int or bool): the number to fill the tensor with.
+         - full (int or list): size of the tensor.
+
+           - value (float or int or bool): the number to fill the tensor with.
+         - ones_like (str or dict): parameter used to determine the size of
+           the tensor filled with the scalar value 1.
+         - ones (int or list): size of the tensor.
+         - zeros_like (str or dict): parameter used to determine the size of
+           the tensor filled with the scalar value 0.
+         - zeros (int or list): size of the tensor.
+         - eye_like (str or dict): parameter used to create a 2-D tensor with
+           ones on the diagonal and zeros elsewhere.
+         - eye (int or list): size of the 2D tensor with ones on the diagonal and
+           zeros elsewhere. The list can only contain 2 integers.
+         - arange (int or list): emulate torch.arange. If a int is provided it is
+           equivalent to torch.arange(x). If a list is provided it is equivalent to
+           torch.arange(x[0], x[1], x[2]). The list can be of size 2 or 3.
+
+         Optional:
+          - dtype (str): the desired data type of returned tensor.
+            Default: if None, infers data type from data.
+          - device (str):  the device of the constructed tensor. If None and data
+            is a tensor then the device of data is used. If None and data is not a
+            tensor then the result tensor is constructed on the CPU.
+          - requires_grad (bool): If autograd should record operations on the returned
+            tensor. Default: False.
+          - nn (bool): If the tensor should be wrapped in a torch.nn.Parameter object.
+
+        **JSON Examples**
+
+        .. code-block:: json
+
+          {
+            "id": "param",
+            "type": "Parameter",
+            "tensor": [1.0, 2.0, 3.0]
+          }
+
+        .. code-block:: json
+
+          {
+            "id": "param2",
+            "type": "Parameter",
+            "full_like": "param",
+            "value": 0.1
+          }
+
+        :example:
+        >>> p_dic = {"id": "parameter", "type": "Parameter", "tensor": [1., 2., 3.]}
+        >>> parameter = Parameter.from_json(p_dic, {})
+        >>> isinstance(parameter, Parameter)
+        True
+        >>> parameter.tensor
+        tensor([1., 2., 3.])
+        >>> ones_dic = {"id": "parameter", "type": "Parameter", "ones_like": p_dic}
+        >>> ones = Parameter.from_json(ones_dic, {})
+        >>> all(ones.tensor == torch.ones(3))
+        True
+
+        .. note::
+            The specification of the tensor loosely follows the way Tensors
+            (full, ones, eye, ...) are constructed:
+            https://pytorch.org/docs/stable/torch.html
+        """
         dtype = get_class(data['dtype']) if 'dtype' in data else None
         device = data.get('device', None)
         kwargs = {'device': device}
@@ -207,14 +308,27 @@ class Parameter(AbstractParameter):
                 kwargs['dtype'] = dtype
             size = data['eye']
             t = torch.eye(size, **kwargs)
+        elif 'eye_like' in data:
+            # input_param should be 1 or 2 dimensional
+            if dtype:
+                kwargs['dtype'] = dtype
+            input_param = process_object(data['eye_like'], dic)
+            t = torch.eye(*input_param.tensor.shape, **kwargs)
+        elif 'arange' in data:
+            if dtype:
+                kwargs['dtype'] = dtype
+            if isinstance(data['arange'], list):
+                t = torch.arange(*data['arange'])
+            else:
+                t = torch.arange(data['arange'])
         else:
             if dtype:
                 kwargs['dtype'] = dtype
             values = data['tensor']
-            if 'dimension' in data:
-                values = np.repeat(values, data['dimension'] / len(values) + 1)
-                values = values[: data['dimension']].tolist()
             t = torch.tensor(values, **kwargs)
+            if 'dimension' in data:
+                dim = data['dimension']
+                t = t.repeat(int(dim / len(values)) + 1)[:dim]
         if 'nn' in data and data['nn']:
             return cls(data['id'], nn.Parameter(t))
         return cls(data['id'], t)
@@ -222,6 +336,22 @@ class Parameter(AbstractParameter):
 
 @register_class
 class TransformedParameter(AbstractParameter, Parametric, collections.abc.Callable):
+    """Class wrapping an AbstractParameter and a torch Transform object.
+
+    The tensor property of this object returns the wrapped parameter tensor
+    transformed with the wrapped transform.
+
+    This class is callable and it returns the log determinant jacobians of the
+    invertible transformation.
+
+    :param id_: object identifier.
+    :type id_: str or None
+    :param x: parameter to transform.
+    :type x: Union[list[AbstractParameter], AbstractParameter]
+    :param transform: torch transform object.
+    :type transform: torch.distributions.Transform
+    """
+
     def __init__(
         self,
         id_: Optional[str],
@@ -244,7 +374,7 @@ class TransformedParameter(AbstractParameter, Parametric, collections.abc.Callab
 
     def __call__(self, *args, **kwargs) -> Tensor:
         if self.need_update:
-            self.apply_transform()
+            self._apply_transform()
             self.need_update = False
 
         return self.transform.log_abs_det_jacobian(self.x.tensor, self._tensor)
@@ -252,7 +382,7 @@ class TransformedParameter(AbstractParameter, Parametric, collections.abc.Callab
     @property
     def tensor(self) -> Tensor:
         if self.need_update:
-            self.apply_transform()
+            self._apply_transform()
             self.need_update = False
         return self._tensor
 
@@ -262,6 +392,9 @@ class TransformedParameter(AbstractParameter, Parametric, collections.abc.Callab
 
     @property
     def requires_grad(self) -> bool:
+        if self.need_update:
+            self._apply_transform()
+            self.need_update = False
         return self._tensor.requires_grad
 
     @requires_grad.setter
@@ -271,11 +404,11 @@ class TransformedParameter(AbstractParameter, Parametric, collections.abc.Callab
     @property
     def shape(self) -> torch.Size:
         if self.need_update:
-            self.apply_transform()
+            self._apply_transform()
             self.need_update = False
         return self._tensor.shape
 
-    def apply_transform(self) -> None:
+    def _apply_transform(self) -> None:
         self._tensor = self.transform(self.x.tensor)
 
     def handle_parameter_changed(self, variable, index, event) -> None:
@@ -312,7 +445,57 @@ class TransformedParameter(AbstractParameter, Parametric, collections.abc.Callab
         self.need_update = True
 
     @classmethod
-    def from_json(cls, data, dic):
+    def from_json(
+        cls, data: dict[str, Any], dic: dict[str, Identifiable]
+    ) -> TransformedParameter:
+        r"""Creates a TransformedParameter object from a dictionary.
+
+        :param dict[str, Any] data: dictionary representation of a transformed
+            parameter object.
+        :param dict[str, Identifiable] dic: dictionary containing torchtree objects
+            keyed by their ID.
+
+        **JSON attributes**:
+
+         Mandatory:
+          - id (str): identifier of object.
+          - x (str or dict): ID or dict representation of a parameter.
+          - transform (str): complete path of the torch transform class,
+            including package and module names.
+
+         Optional:
+          - parameters (dic): parameters of torch transform.
+
+        **JSON Example**
+
+        .. code-block:: json
+
+          {
+            "id": "positive",
+            "type": "TransformedParameter",
+            "transform": "torch.distributions.ExpTransform",
+            "x" {
+              "id": "unconstrained",
+              "type": "Parameter",
+              "tensor": -1.0
+            }
+          }
+
+        :example:
+        >>> tensor = torch.tensor([1.,2.])
+        >>> p_dic = {"id": "parameter", "type": "Parameter", "tensor": tensor.tolist()}
+        >>> t_dic =  {"id": "t", "type": "TransformedParameter", "x": p_dic,
+        ... "transform": "torch.distributions.ExpTransform"}
+        >>> transformed = TransformedParameter.from_json(t_dic, {})
+        >>> isinstance(transformed, TransformedParameter)
+        True
+        >>> exp_transform = torch.distributions.ExpTransform()
+        >>> tensor2 = exp_transform(tensor)
+        >>> all(transformed.tensor == tensor2)
+        True
+        >>> all(transformed() == exp_transform.log_abs_det_jacobian(tensor, tensor2))
+        True
+        """
         # parse transform
         klass = get_class(data['transform'])
         signature_params = list(inspect.signature(klass.__init__).parameters)
@@ -341,20 +524,20 @@ class TransformedParameter(AbstractParameter, Parametric, collections.abc.Callab
 
 @register_class
 class ViewParameter(AbstractParameter, ParameterListener):
+    r"""Class representing a view of another parameter.
+
+    :param id_: ID of object.
+    :type id_: str or None
+    :param Parameter parameter: parameter that ViewParameter wrap.
+    :param indices: indices used on parameter
+    """
+
     def __init__(
         self,
         id_: Optional[str],
         parameter: Parameter,
         indices: Union[int, slice, Tensor],
     ) -> None:
-        r"""
-        Class representing a view of another parameter.
-
-        :param id_: ID of object
-        :type id_: str or None
-        :param Parameter parameter: parameter that ViewParameter wrap
-        :param indices: indices used on parameter
-        """
         AbstractParameter.__init__(self, id_)
         self.parameter = parameter
         self.indices = indices
@@ -400,7 +583,7 @@ class ViewParameter(AbstractParameter, ParameterListener):
 
     @property
     def shape(self) -> torch.Size:
-        return self.parameter.tensor.shape
+        return self.parameter.tensor[..., self.indices].shape
 
     @property
     def dtype(self) -> torch.dtype:
