@@ -18,10 +18,39 @@ REGISTERED_CLASSES = {}
 class TensorEncoder(json.JSONEncoder):
     def default(self, obj):
         if isinstance(obj, torch.Tensor):
-            return {'values': obj.tolist(), 'type': str(obj.dtype)}
+            dic = {
+                "type": "torch.Tensor",
+                "values": obj.tolist(),
+                "dtype": str(obj.dtype),
+            }
+            if isinstance(obj, torch.nn.Parameter):
+                dic["nn"] = True
+            return dic
         return json.JSONEncoder.default(self, obj)
 
 
+class TensorDecoder(json.JSONDecoder):
+    def __init__(self, *args, **kwargs):
+        json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
+
+    def object_hook(self, dic):
+        if "type" in dic and dic["type"] == "torch.Tensor":
+            del dic["type"]
+            if "dtype" in dic:
+                dic["dtype"] = getattr(torch, dic["dtype"].split(".")[-1])
+            nn = False
+            if "nn" in dic:
+                del dic["nn"]
+                nn = True
+            values = dic["values"]
+            del dic["values"]
+            tensor = torch.tensor(values, **dic)
+            if nn:
+                tensor = torch.nn.Parameter(tensor)
+            return tensor
+        return dic
+
+
 def as_tensor(dct, dtype=torch.float64):
     if 'type' in dct and dct['type'].startswith('torch'):
         return torch.tensor(dct['values'], dtype=dtype)
@@ -84,7 +113,7 @@ def tensor_rand(distribution, shape, dtype=None, device=None, requires_grad=Fals
     return tensor
 
 
-def get_class(full_name: str) -> any:
+def get_class(full_name: str) -> type:
     if full_name in REGISTERED_CLASSES:
         return REGISTERED_CLASSES[full_name]
 
@@ -100,57 +129,76 @@ class JSONParseError(Exception):
     ...
 
 
-def process_objects(data, dic):
+def process_objects(data, dic, force_list=False, key=None):
+    if key is not None and key not in data:
+        return [] if force_list else None
+    elif key is not None and key in data:
+        data = data[key]
+
     if isinstance(data, list):
         return [process_object(obj, dic) for obj in data]
     else:
-        return process_object(data, dic)
+        obj = process_object(data, dic)
+        if force_list:
+            return [obj]
+        else:
+            return obj
+
+
+def process_object_with_key(key, data, dic, default=None):
+    if key in data:
+        return process_object(data[key], dic)
+    else:
+        return default
 
 
 def process_object(data, dic):
     if isinstance(data, str):
         # for references such as branches.{0:3}
         try:
-            if '{' in data:
-                stem, indices = data.split('{')
-                start, stop = indices.rstrip('}').split(':')
+            if "{" in data:
+                stem, indices = data.split("{")
+                start, stop = indices.rstrip("}").split(":")
                 for i in range(int(start), int(stop)):
                     obj = dic[stem + str(i)]
             else:
                 obj = dic[data]
         except KeyError:
             raise JSONParseError(
-                'Object with ID `{}\' not found'.format(data)
+                "Object with ID `{}\' not found".format(data)
             ) from None
     elif isinstance(data, dict):
-        id_ = data['id']
+        try:
+            id_ = data["id"]
+        except KeyError as e:
+            logging.error(e)
+            if "type" in data:
+                type_ = data["type"]
+                raise JSONParseError(
+                    f"Missing `id' key for object of type `{type_}'"
+                ) from None
+            else:
+                raise JSONParseError("Missing `id' and `type' keys") from None
+
         if id_ in dic:
-            raise JSONParseError('Object with ID `{}\' already exists'.format(id_))
-        if 'type' not in data:
-            raise JSONParseError(
-                'Object with ID `{}\' does not have a type'.format(id_)
-            )
+            raise JSONParseError(f"Object with ID `{id_}' already exists")
+        if "type" not in data:
+            raise JSONParseError(f"Object with ID `{id_}' does not have a type")
 
         try:
-            klass = get_class(data['type'])
+            klass = get_class(data["type"])
         except ModuleNotFoundError as e:
-            raise JSONParseError(
-                str(e) + " in object with ID '" + data['id'] + "'"
-            ) from None
+            raise JSONParseError(str(e) + f" in object with ID `{id_}'") from None
         except AttributeError as e:
-            raise JSONParseError(
-                str(e) + " in object with ID '" + data['id'] + "'"
-            ) from None
+            raise JSONParseError(str(e) + f" in object with ID `{id_}'") from None
         except ValueError as e:
-            raise JSONParseError(
-                str(e) + " in object with ID '" + data['id'] + "'"
-            ) from None
+            raise JSONParseError(str(e) + f" in object with ID `{id_}'") from None
 
         obj = klass.from_json_safe(data, dic)
         dic[id_] = obj
     else:
         raise JSONParseError(
-            'Object is not valid (should be str or object)\nProvided: {}'.format(data)
+            "Object is not valid (should be str or object)\nProvided: {}".format(data)
         )
     return obj
 
@@ -215,12 +263,25 @@ def validate(data, rules):
 
 
 def remove_comments(obj):
+    """Remove comments in dictionary representation of objects.
+
+    - A key starting with an underscore results in the key/value pair to be removed.
+    - A dictionary with key equal to *ignore* and value set to *True* results in its
+      removal.
+    """
     if isinstance(obj, list):
-        for element in obj:
-            remove_comments(element)
+        for i in range(len(obj) - 1, -1, -1):
+            if isinstance(obj[i], dict) and "ignore" in obj[i] and obj[i]["ignore"]:
+                del obj[i]
+            else:
+                remove_comments(obj[i])
     elif isinstance(obj, dict):
         for key in list(obj.keys()).copy():
-            if not key.startswith('_'):
+            if not key.startswith('_') and not (
+                isinstance(obj[key], dict)
+                and "ignore" in obj[key]
+                and obj[key]["ignore"]
+            ):
                 remove_comments(obj[key])
             else:
                 del obj[key]
@@ -305,8 +366,7 @@ def update_parameters(json_object, parameters) -> None:
 
 
 def print_graph(g: torch.Tensor, level: int = 0) -> None:
-    r"""
-    Print computation graph.
+    r"""Print computation graph.
 
     :param torch.Tensor g: a tensor
     :param level: indentation level
@@ -326,7 +386,7 @@ class AlternativeAttributeError(Exception):
 
     @classmethod
     def wrapper(err_type, f):
-        """wraps a function to reraise an AttributeError as the alternate
+        """Wraps a function to reraise an AttributeError as the alternate
         type."""
 
         @functools.wraps(f)
