@@ -1,3 +1,5 @@
+"""The distributions package contains parameterizable probability distributions
+and transforms."""
 from .distributions import Distribution
 from .inverse_gamma import InverseGamma
 from .multivariate_normal import MultivariateNormal
