@@ -1,9 +1,13 @@
-from typing import Union
+"""Scale mixture of Normal distributions."""
+from __future__ import annotations
+
+from typing import Any, Union
 
 from torch import Size, Tensor
 from torch.distributions import Normal
 
 from torchtree.core.abstractparameter import AbstractParameter
+from torchtree.core.identifiable import Identifiable
 from torchtree.core.model import CallableModel
 from torchtree.core.utils import process_object, register_class
 from torchtree.typing import ID
@@ -11,9 +15,10 @@ from torchtree.typing import ID
 
 @register_class
 class ScaleMixtureNormal(CallableModel):
-    """Scale mixture of Normal distributions.
+    r"""Scale mixture of Normal distributions.
 
-    Regularized when a slab width parameter or scalar is provided [piironen2017]_
+    Regularized when a slab width parameter or scalar is provided
+    :footcite:p:`piironen2017sparsity`.
 
     :param id_: ID of object
     :param loc: mean of the distribution
@@ -22,8 +27,7 @@ class ScaleMixtureNormal(CallableModel):
     :param gamma: local scale
     :param slab: slab width
 
-    .. [piironen2017] Piironen and Vehtari 2017. Sparsity information and regularization
-     in the horseshoe and other shrinkage priors.
+    .. footbibliography::
     """
 
     def __init__(
@@ -59,8 +63,7 @@ class ScaleMixtureNormal(CallableModel):
             self.x.tensor
         )
 
-    @property
-    def sample_shape(self) -> Size:
+    def _sample_shape(self) -> Size:
         return self.x.tensor.shape[:-1]
 
     def handle_model_changed(self, model, obj, index) -> None:
@@ -81,7 +84,41 @@ class ScaleMixtureNormal(CallableModel):
         return model
 
     @classmethod
-    def from_json(cls, data, dic):
+    def from_json(
+        cls, data: dict[str, Any], dic: dict[str, Identifiable]
+    ) -> ScaleMixtureNormal:
+        r"""Creates a ScaleMixtureNormal object from a dictionary.
+
+        :param dict[str, Any] data: dictionary representation of a
+            ScaleMixtureNormal object.
+        :param dict[str, Identifiable] dic: dictionary containing torchtree objects
+            keyed by their ID.
+
+        **JSON attributes**:
+
+         Mandatory:
+          - id (str): unique string identifier.
+          - x (dict or str): parameter.
+          - loc (dict or str or float): location parameter.
+          - scale (dict or str): global scale parameter.
+          - local_scale (dict or str): local scale parameter.
+
+         Optional:
+          - slab (dict or str or float): slab parameter
+
+        :example:
+        >>> x = {"id": "x", "type": "Parameter", "tensor": [1., 2.]}
+        >>> loc = {"id": "loc", "type": "Parameter", "tensor": [1.]}
+        >>> global_scale = {"id": "global", "type": "Parameter", "tensor": [1.]}
+        >>> local_scale = {"id": "local", "type": "Parameter", "tensor": [0.1, 0.2]}
+        >>> mixture_dic = {"id": "mixture", "x": x, "loc": loc,
+        ...     "global_scale": global_scale, "local_scale": local_scale}
+        >>> mixture = ScaleMixtureNormal.from_json(mixture_dic, {})
+        >>> isinstance(mixture, ScaleMixtureNormal)
+        True
+        >>> isinstance(mixture(), Tensor)
+        True
+        """
         id_ = data['id']
         x = process_object(data['x'], dic)
         loc = (
