@@ -2,6 +2,8 @@ from torchtree.evolution.substitution_model.amino_acid import LG, WAG
 from torchtree.evolution.substitution_model.codon import MG94
 from torchtree.evolution.substitution_model.general import (
     EmpiricalSubstitutionModel,
+    GeneralJC69,
+    GeneralNonSymmetricSubstitutionModel,
     GeneralSymmetricSubstitutionModel,
 )
 from torchtree.evolution.substitution_model.nucleotide import GTR, HKY, JC69
@@ -11,7 +13,9 @@ __all__ = [
     'HKY',
     'GTR',
     'EmpiricalSubstitutionModel',
+    'GeneralJC69',
     'GeneralSymmetricSubstitutionModel',
+    'GeneralNonSymmetricSubstitutionModel',
     'LG',
     'WAG',
     'MG94',
