@@ -1,7 +1,7 @@
 import torch
 from torch.distributions import Transform
 
-from ..math import soft_max
+from torchtree.ops.smooth import smooth_max
 
 
 class GeneralNodeHeightTransform(Transform):
@@ -112,7 +112,7 @@ class DifferenceNodeHeightTransform(Transform):
     sign = +1
 
     def __init__(
-        self, tree_model: 'TimeTreeModel', k: float = 1.0, cache_size=0  # noqa: F821
+        self, tree_model: 'TimeTreeModel', k: float = 0.0, cache_size=0  # noqa: F821
     ) -> None:
         super().__init__(cache_size=cache_size)
         self.tree = tree_model
@@ -121,7 +121,7 @@ class DifferenceNodeHeightTransform(Transform):
         if self.k <= 0:
             self.max = lambda input: torch.max(input, dim=-1, keepdim=True)[0]
         else:
-            self.max = lambda input: soft_max(input, self.k, dim=-1, keepdim=True)
+            self.max = lambda input: smooth_max(input, self.k, dim=-1, keepdim=True)
 
     def _call(self, x: torch.Tensor) -> torch.Tensor:
         """Transform node height differences to internal node heights."""
@@ -141,17 +141,25 @@ class DifferenceNodeHeightTransform(Transform):
         heights = list(
             self.tree.sampling_times.expand(y.shape[:-1] + (-1,)).split(1, -1)
         ) + list(y.split(1, -1))
-        for node, left, right in self.tree.postorder:
-            x[node - self.taxa_count] = (
-                heights[node]
-                - torch.logsumexp(
-                    torch.cat((heights[left] * self.k, heights[right] * self.k), -1),
-                    dim=-1,
-                    keepdim=True,
+        if self.k > 0:
+            for node, left, right in self.tree.postorder:
+                x[node - self.taxa_count] = (
+                    heights[node]
+                    - torch.logsumexp(
+                        torch.cat(
+                            (heights[left] * self.k, heights[right] * self.k), -1
+                        ),
+                        dim=-1,
+                        keepdim=True,
+                    )
+                    / self.k
+                )
+        else:
+            for node, left, right in self.tree.postorder:
+                x[node - self.taxa_count] = heights[node] - torch.max(
+                    heights[left], heights[right]
                 )
-                / self.k
-            )  # - torch.max(heights[left], heights[right])
         return torch.cat(x, -1)
 
     def log_abs_det_jacobian(self, x, y):
-        return torch.zeros(x.shape[:-1])
+        return torch.zeros(x.shape[:-1], dtype=x.dtype, device=x.device)
