@@ -240,9 +240,8 @@ class RealNVP(DistributionModel):
     def batch_shape(self) -> torch.Size:
         return self.base_dist.batch_shape
 
-    @property
-    def sample_shape(self) -> torch.Size:
-        return self.base.sample_shape
+    def _sample_shape(self) -> torch.Size:
+        return self.base_dist.sample_shape
 
     def parameters(self) -> list[AbstractParameter]:
         return self.net_parameters.parameters()
@@ -262,4 +261,4 @@ class RealNVP(DistributionModel):
         real_nvp = cls(id_, x, base, n_blocks, hidden_size, n_hidden)
         if 'state' in data:
             real_nvp.net.load_state_dict(torch.load(data['state']))
-        return cls(id_, x, base, n_blocks, hidden_size, n_hidden)
+        return real_nvp
