@@ -7,11 +7,11 @@ from typing import Optional, Union
 import torch
 from torch import Tensor, nn
 
-from ..core.abstractparameter import AbstractParameter
-from ..core.container import Container
-from ..core.model import CallableModel
-from ..core.utils import get_class, process_objects, register_class
-from ..typing import ID, OrderedDictType
+from torchtree.core.abstractparameter import AbstractParameter
+from torchtree.core.container import Container
+from torchtree.core.model import CallableModel
+from torchtree.core.utils import get_class, process_objects, register_class
+from torchtree.typing import ID, OrderedDictType
 
 
 @register_class
@@ -37,17 +37,13 @@ class Module(CallableModel):
 
     @property
     def module(self) -> nn.Module:
-        r"""
-        :return: a pytorch Module object.
-        :rtype: torch.nn.Module
-        """
+        """returns torch.nn.Module"""
         return self._module
 
     def _call(self, *args, **kwargs) -> Tensor:
         return self._module()
 
-    @property
-    def sample_shape(self) -> torch.Size:
+    def _sample_shape(self) -> torch.Size:
         raise NotImplementedError
 
     def to(self, *args, **kwargs) -> None:
