@@ -50,8 +50,7 @@ class CUBO(CallableModel):
     def handle_parameter_changed(self, variable, index, event):
         pass
 
-    @property
-    def sample_shape(self) -> torch.Size:
+    def _sample_shape(self) -> torch.Size:
         return self.q.sample_shape
 
     @classmethod
